/*
* ClmChrono 1.1.1 by Franois CREVOLA - francois@crevola.org
*  License : GPL V2.0 (excluding the ClmChrono.png and ClmChronoSplash.png files which are (c) CourirLeMonde.org)
*/

import javax.microedition.lcdui.*;
import javax.microedition.rms.*;
import javax.microedition.io.*;
import java.io.*;
import java.util.*;

class ClmEngine extends Canvas {

  private int _height;
  private int _width;
  private int choixPolice;
  private boolean _painting = false;
  public long _startTimeStamp;
  public long _stopTimeStamp ; 
  private long deltaTOP; 
  public long  lastTOP;  
  public boolean started; 
  public boolean requestSent;
  private int distance = 21100; // default distance in meter
  private int lapDistance = 1000; // default lap distance in meter 
  public String defaultExportURL = "http://www.crevola.org/clmpost.php";
  private String exportURL = defaultExportURL;
  private String defaultPseudo = ""; // default pseudo
  private String pseudo = defaultPseudo;
  private int _nbLap; // nb of lap done
  private ClmChrono _parent;
  public Vector _laps;
  
  public ClmEngine()
  {
    _height = getHeight(); // hauteur totale de l'cran
    _width = getWidth(); // largeur totale de l'cran
    readRecords();
  
    init();
  }

  public void init() {
    _laps = new Vector();
    _startTimeStamp = new Date().getTime();
    _stopTimeStamp = _startTimeStamp;
    deltaTOP = 0;
    lastTOP = deltaTOP;
    started = false;
    _nbLap = 0; // nb of lap done
  }

  public void setFullDistance(int i) {
    distance = i;
  }

  public void setParentClass(ClmChrono clm) {
    _parent = clm;
  }

  public ClmChrono getParentClass() {
    return _parent;
  }

  public void stopChrono(long stopTimeStamp) {
      if (started) {
      	removeCommand(_parent.stopCommand);
        addCommand(_parent.startCommand);
  	    removeCommand(_parent.lapCommand);
        addCommand(_parent.optionCommand);
        _stopTimeStamp = stopTimeStamp;
        started = false;
      }
  }

  public void initChronoCommand() {
    removeCommand(_parent.exportCommand);
    removeCommand(_parent.resetCommand);
    addCommand(_parent.startCommand);
    addCommand(_parent.optionCommand);
  }

  public void displayReset() {
     	removeCommand(_parent.startCommand);
      removeCommand(_parent.optionCommand); 
      addCommand(_parent.resetCommand);
      addCommand(_parent.exportCommand);
  }

  public int getFullDistance() {
    return distance;
  }

  public void setChoixPolice(int i) {
    switch (i) {
      case 0:
        choixPolice = Font.SIZE_SMALL;
        break;
      case 1:
        choixPolice = Font.SIZE_MEDIUM;
        break;
      case 2:
        choixPolice = Font.SIZE_LARGE;
        break;
    }
  }

  public int getChoixPolice() {
    switch (choixPolice) {
      case Font.SIZE_SMALL:
        return 0;
      case Font.SIZE_MEDIUM:
        return 1;
      case Font.SIZE_LARGE:
        return 2;
    }
    return 1;
  }

  public void setLapDistance(int i) {
    lapDistance = i;
  }

  public int getLapDistance() {
    return lapDistance;
  }

  public void setPseudo(String p) {
    pseudo = p;
  }

  public String getPseudo() {
    return pseudo;
  }

  public void setExportURL(String p) {
    exportURL = p;
  }

  public String getExportURL() {
    return exportURL;
  }

  // appel par le timer toutes les 20 ms
  public void TestAndMove() {
    
    if (!_painting) {
      repaint(); // appel  la methode paint() pour afficher les changements
    }
    return;

  }

  protected void paint(Graphics g) {

    long _distanceDone = _nbLap * lapDistance;
    if (_distanceDone>=distance) {
      _distanceDone = distance;  
    }

    long currentTimeStamp = new Date().getTime();
    if (!started) currentTimeStamp = _stopTimeStamp;

    long delta = currentTimeStamp  - _startTimeStamp;
    long lap = delta - deltaTOP;
    long vitesse = 0;
    
    _painting = true;
    Font font = Font.getFont(Font.FACE_SYSTEM,Font.STYLE_PLAIN,choixPolice);
    int fontHeight = font.getHeight();
    g.setFont(font);
    g.setColor(255, 255, 255); // Blanc
    g.fillRect(0,0,_width,_height);
    g.setColor(0,0,0); // Noir
    g.drawString(Date2String(deltaTOP-lastTOP)+" (last Lap)",0,fontHeight*0,Graphics.TOP | Graphics.LEFT);
    g.drawString(Date2String(lap)+" (current Lap)",0,fontHeight*1,Graphics.TOP | Graphics.LEFT);
    g.drawString(Date2String(delta)+" (Total)",0,fontHeight*2,Graphics.TOP | Graphics.LEFT); 
    g.drawString(new Long(_distanceDone).toString()+" m / "+distance+" m",0,fontHeight*3,Graphics.TOP | Graphics.LEFT); 

    if (_distanceDone > 0) {
      long preview = ( distance * deltaTOP ) / _distanceDone ; 
      g.drawString(Date2String(preview),0,fontHeight*4,Graphics.TOP | Graphics.LEFT); 
      vitesse = (_distanceDone*3600*1000)/deltaTOP;
      g.drawString(Vitesse2String(vitesse)+" km/h",0,fontHeight*5,Graphics.TOP | Graphics.LEFT); 
    }
    
    _painting = false;
    return;
  } 
  
  protected String Date2String(long timestamp) {

    Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
    cal.setTime(new Date(timestamp));
    String today = extendLeft(cal.get(Calendar.HOUR_OF_DAY)+"",2) + ":" + extendLeft(cal.get(Calendar.MINUTE)+"",2) + ":" + extendLeft(cal.get(Calendar.SECOND)+"",2) +","+ extendLeft(cal.get(Calendar.MILLISECOND)+"",3).substring(0,2);
    return today;
  }

  protected String Vitesse2String(long vitesse) {
    String vitesseKmH = extendLeft(vitesse+"",5);
    return (vitesse/1000)+"."+vitesseKmH.substring(2,4);
  }

  protected String extendLeft(String str,int nb) {

    while (str.length()<nb) {
      str = "0" + str;
    }
    return str;
  }

  // lorsque une touche est appuye 
  public void handleCommand(int keyCode) { // was keyPressed(...)
    boolean top = false;
    long now = new Date().getTime();   
    
    if (started) {
      if (keyCode==Canvas.KEY_STAR) { // gauche
        top = true;
        _nbLap--;
      } else if (keyCode==Canvas.KEY_POUND) { // droite
        top = true;
        _nbLap++;
      }
    }
  
    if (top) {
      lastTOP = deltaTOP; // Save it before change it !
      deltaTOP = now - _startTimeStamp;
      _laps.addElement(new Long(deltaTOP));
      /*
      System.out.println(deltaTOP);
      System.out.println(_laps);
      */
    }

    if (_nbLap * lapDistance >= distance) {
      stopChrono(now);
      displayReset();
    }

  }

 public void saveConfig() {
  RecordStore rs = null;
try {
  
  try {
     // Si le record store existe, on l'ouvre
     rs = RecordStore.openRecordStore("ClmChronoRMS",false); 
  } catch (Exception ex) {
     // Si le record store n'existe pas, on le cr avec 3 lements vide pour l'instant ...
     rs = RecordStore.openRecordStore("ClmChronoRMS",true); 
     rs.addRecord(null,0,0);
     rs.addRecord(null,0,0);
     rs.addRecord(null,0,0);
     rs.addRecord(null,0,0);
     rs.addRecord(null,0,0);
  }
   
  // Lap
  String appt = lapDistance+"";
  byte bytes[] = appt.getBytes();
  rs.setRecord(1,bytes,0,bytes.length);
  
  // Distance
  appt = distance+"";
  bytes = appt.getBytes();
  rs.setRecord(2,bytes,0,bytes.length);
  
  // FontIndex
  appt = getChoixPolice()+"";
  bytes = appt.getBytes();
  rs.setRecord(3,bytes,0,bytes.length);

  // Pseudo
  appt = getPseudo()+"";
  bytes = appt.getBytes();
  rs.setRecord(4,bytes,0,bytes.length);

  // Export Server URL
  appt = getExportURL()+"";
  bytes = appt.getBytes();
  rs.setRecord(5,bytes,0,bytes.length);

  rs.closeRecordStore();
  
  } catch (Exception ex) {
    System.err.println("oh oh");
  }

 }

  public void readRecords() {

    try {

      RecordStore recStore = RecordStore.openRecordStore("ClmChronoRMS",false); 
      byte[] recData = new byte[5]; 
      int len;
      String str = "";
      
        recData = new byte[recStore.getRecordSize(1)];
        len = recStore.getRecord(1, recData, 0);
        str = new String(recData, 0, len);
        setLapDistance(Integer.valueOf(str).intValue());

        recData = new byte[recStore.getRecordSize(2)];
        len = recStore.getRecord(2, recData, 0);
        str = new String(recData, 0, len);
        setFullDistance(Integer.valueOf(str).intValue());

        recData = new byte[recStore.getRecordSize(3)];
        len = recStore.getRecord(3, recData, 0);
        str = new String(recData, 0, len);
        setChoixPolice(Integer.valueOf(str).intValue());

        recData = new byte[recStore.getRecordSize(4)];
        len = recStore.getRecord(4, recData, 0);
        str = new String(recData, 0, len);
        setPseudo(str);

        recData = new byte[recStore.getRecordSize(5)];
        len = recStore.getRecord(5, recData, 0);
        str = new String(recData, 0, len);
        setExportURL(str);

    } catch (Exception e) { // A la moindre exception, on prend des valeurs par dfaut !!
      setLapDistance(1000);
      setFullDistance(21100);
      setChoixPolice(1);
      setPseudo(defaultPseudo);
      setExportURL(defaultExportURL);
      System.err.println(e.toString());
    }
  
  }  
  
}
